/*
 * Copyright (C) 2000-2001  Ken McCrary
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Email: jkmccrary@yahoo.com
 */

package estella.lib;
import  estella.lib.Log;
import  estella.lib.Constantes;

import java.io.DataOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;


/**
* Gerencia descritor
* @author Cesar Kallas
* @see 02/12/2005
*/
public class Descritor
{
  private static short[] clientID = null;
  private static long bootTime;
  private static long counter = 0x00000000;
  private Log LOG;
	private Constantes CON;
  private byte tipo, ttl, hops;
	private byte[] payload;
	private byte[] DESCRITOR;
	private int payload_length;
		
	/**
	* Instancia uma nova classe do tipo DID
	* @author Cesar Kallas
	*/
	public Descritor(byte tipo, byte ttl, byte[] payload)
	{
		LOG = new Log();
		CON = new Constantes();
		
		bootTime = System.currentTimeMillis();
		this.tipo = tipo;
		this.ttl = ttl;
		
		
		if(tipo != CON.PING) {
			this.payload_length = (int) payload.length;
			this.payload = new byte[payload.length];	
			for(int i=0; i<payload.length; i++)
				this.payload[i] = payload[i];
		}
		else
			this.payload_length = 0x00000000;

		this.DESCRITOR = new byte[23 + payload_length];
	}
	
	/**
	* Generate something remotely resembling a windows guid
	* Essa funcao pertence originalmente ao software jTella versao 0.7 - cedida sobre GPL 
	* @author KenMcCrary
	*/
  private static short[] id()
  {
    // TODO clean this up
    short[] data = new short[16];
    int arrayIndex = 15;
    for (int i = 0; i < 8; i++) {
       // Mask to process one byte at a time
       long mask = 0X00000000000000FF;

       // Adjust the mask to look at the appropriate byte
       mask = mask << (8 * i);

       long result = bootTime & mask;

       // Move the byte we interested in to the low order
       result = result >> (8 * i );

       // copy the processed byte
       data[arrayIndex--] = (short)result;
    }

    counter++;
    arrayIndex = 7;
    for (int i = 0; i < 8; i++) {
      long mask = 0x00000000000000FF;
      mask = mask << (8 * i);
      long result = counter & mask;

      result = result >> (8 * i );
      data[arrayIndex--] = (short)result;
    }
    return data;
  }
	
	/**
	* Gera um novo descritor, incluindo o numero de identificacao<br>
	* esse descritor depende dos parametros passados na instanciacao nessa classe
	* @return byte[] - retorna um array de bytes com o descritor
	*/
	private byte[] gera()
	{
		int i;
		short[] ID = id();
		
		// Descriptor ID (0~15)
		for(i=0; i<15; i++)
			DESCRITOR[i] = (byte)ID[i];
		
		// Payload Descriptor (16)
		DESCRITOR[++i] = tipo;
		
		// TTL (17)
		DESCRITOR[++i] = ttl;
		
		// Hops (18)
		DESCRITOR[++i] = 0;
		
		// Payload length (19~22)
		DESCRITOR[++i] = (byte)(0x000000FF & payload_length);						// byte 19
		DESCRITOR[++i] = (byte)((0x0000FF00 & payload_length) >> 8);		// byte 20
		DESCRITOR[++i] = (byte)((0x00FF0000 & payload_length) >> 16);		// byte 21
		DESCRITOR[++i] = (byte)((0xFF000000 & payload_length) >> 24);		// byte 22
		
		// Copia o Payload para o descritor (23~payload_length)
		for(; i<payload_length; i++)
			DESCRITOR[i] = payload[i];
		
		return DESCRITOR;
	}
	
	/**
	* Retorna o descritor gerado
	* @return byte[] - retorna um array de bytes com o descritor, se ele nao foi criado, o retorno sera null
	*/
	public byte[] retorna()
	{
		return DESCRITOR;
	}
	
	/**
	* TTL - time to live
	* @return byte - retorna o ttl do descritor
	*/
	public byte ttl()
	{
		return ttl;
	}
	
	/**
	* TTL -time to live
	* @param ttl - seta o ttl com o byte passado
	*/
	public void ttl(byte ttl)
	{
		this.ttl = ttl;
	}
	
	/**
	* HOPS - Seta o numero de vezes que o descritor foi roteado (passado por outros servents)
	* @param hops - byte com o hops a ser setado
	*/
	public void hops(byte hops)
	{
		this.hops = hops;
	}
	
	/**
	* HOPS - Retorna o numero de vezes que o descritor foi roteado (passado por outros servents)
	* @return  hops - byte com o hops
	*/
	public byte hops()
	{
		return hops;
	}
}
