/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

/*
	<<		op1 << op2		Shift bits of op1 left by distance op2; fills with zero bits on the right-hand side
	>>		op1 >> op2		Shift bits of op1 right by distance op2; fills with highest (sign) bit on the left-hand side
	>>>		op1 >>> op2		Shift bits of op1 right by distance op2; fills with zero bits on the left-hand side
*/
package estella.lib;

import  java.io.ByteArrayOutputStream;
import	java.io.DataOutputStream;

/**
* Descriptor Payloads (little-endian)
* @author Cesar Kallas
* @since 21/11/2005
*/
public class Payload
{
  private ByteArrayOutputStream bytes;
  private DataOutputStream PONG, QUERY, QUERYHITS;
	
	/**
	* Payload PONG
	* Estrutura:
	* 	|porta|host|n_arq_comp|tam_arq_comp|
	*	|--|-----|-----|-----|          ---> |2|4|4|4|  (bytes)
	* Todos inteiros em little-endian
	* @param porta	- inteiro com a porta 
	* @param host - String com o host
	* @param n_arq_comp - inteiro com o numero de arquivos compartilhados
	* @param tam_arq_comp - inteiro com o tamanho total desses arquivos compartilhados
	* @return byte[] - vetor com os bytes do payload
	*/
	public byte[] pong(int porta, String host, int n_arq_comp, int tam_arq_comp)
	{
		bytes = new ByteArrayOutputStream();
		PONG = new DataOutputStream(bytes);
		
		// Porta (0~1)
		PONG.write(0x00FF & port);				// byte 1 
		PONG.write((0xFF00 & port) >> 8);	// byte 2
		
		// Host (2~5)
		StringTokenizer thost = new StringTokenizer(host, ".");
		PONG.write(Integer.parseInt(thost.nextToken()));	// byte 2
		PONG.write(Integer.parseInt(thost.nextToken()));	// byte 3
		PONG.write(Integer.parseInt(thost.nextToken()));	// byte 4
		PONG.write(Integer.parseInt(thost.nextToken()));	// byte 5
		
		// Numero de arquivos compartilhados (6-9)
		PONG.write(x000000FF & n_arq_comp);						// byte 6
    PONG.write((0x0000FF00 & n_arq_comp) >> 8);		// byte 7
		PONG.write((0x00FF0000 & n_arq_comp) >> 16);	// byte 8
		PONG.write((0xFF000000 & n_arq_comp) >> 24);	// byte 9
		
		// Tamanho total dos arquivos compartilhados (10-13)
		PONG.write(x000000FF & tam_arq_comp);						// byte 10
    PONG.write((0x0000FF00 & tam_arq_comp) >> 8);		// byte 11
		PONG.write((0x00FF0000 & tam_arq_comp) >> 16);	// byte 12
		PONG.write((0xFF000000 & tam_arq_comp) >> 24);	// byte 13
		
		return(PONG.toByteArray());
	}
}
